import dabax
from numpy import *
from string import atof, split
import Ifeffit
iff = Ifeffit.Ifeffit()

import reader
import writer
import diffkk
import ppmxml
import feff

# Calculation of the optical indexes
# const_beta = ((h*c)^2*re*Na)/2pi
# const_mu = 2*h*c*re*Na
# const_energy = (h/2pi)^2/2me
# n = 1 - delta - i*beta
# indexFromFile = [energy, (1-delta), beta]
# beta = (Ni*const_beta*f2)/(Awi*(en**2)); delta = (Ni*const_beta*(f0 + f1))/(Awi*(en**2))
# mu = (Ni*const_mu*f2)/(Awi*en);
# f2_edge = (en*dmu*chi*Awi)/(const_mu*Ni) + f2_0
# E = E0 + const_energy*k^2


const_beta = 415.185543381
const_mu = 42080663.068719
const_energy = 3.809984069

Table_f0= dabax.Dabax_f0_Table("f0_WaasKirf.dat")
Table_f1f2= dabax.Dabax_f1f2_Table("f1f2_Sasaki.dat")


class CalcEdge:
    def __init__(self, inp_file, E0, l_name):
        self.E0 = E0
        self.e_list = arange((self.E0-300.0),(self.E0+300.0),1.0)
        self.mui = []
        self.f2 = []
        
        self.xmu = zeros(len(self.e_list))
        rd = reader.ReadXml()
        dic = rd.readInp(inp_file)
        stack, rep = rd.manageSTACK(dic)
        for i in range(len(stack)):
            if stack[i][0] == l_name:
                self.currentLayer = stack[i][2:]
                self.currentDensity = atof(split(stack[i][1][2])[1])
#         append the f2s and calculates the other parameters
        for b in range(len(self.currentLayer)):
            element = Table_f1f2.Element(self.currentLayer[b][0])
            self.f2.append(imag(element.f1f2Energy(self.e_list)))
#         build the xmu adding all the terms
        for c in range(len(self.currentLayer)):
            Pmi = dabax.AtomicProperties(self.currentLayer[c][0],'AtomicMass')
            self.currentLayer[c][1] = split(self.currentLayer[c][1])
            mui = (atof(self.currentLayer[c][1][0])*const_mu*self.f2[c]*self.currentDensity)/(Pmi*self.e_list)
            for y in range(len(mui)):
                self.xmu[y] = self.xmu[y]+ mui[y]
        self.xmu = array(self.xmu)
    def Edge(self):
        iff.put_array('a.en', self.e_list)
        iff.put_array('a.xmu', self.xmu)
        iff.put_scalar('econzero', self.E0)
        iff.ifeffit("pre_edge(a.en,a.xmu,e0=econzero)")
        self.edge = iff.get_scalar("edge_step")
        iff.ifeffit("erase @arrays, @scalars")
        return self.edge
    


class Edge_f2:
    def __init__(self, feff_file, conta, el_name, inp_file, E0, l_name):
        self.E0 = E0
        info_path = "path(index="+str(conta)+",file="+feff_file+",s02=1,sigma2=0)"
        iff.ifeffit(info_path)
        get = "get_path("+str(conta)+")"
        iff.ifeffit(get)
        ff2 = "ff2chi("+str(conta)+",do_all)"
        iff.ifeffit(ff2)
        iff.ifeffit("maxk=ceil(feff.k)")
        iff.ifeffit("mink=floor(feff.k)")
        self.mink = iff.get_scalar("mink")
        self.maxk = iff.get_scalar("maxk")
        self.reff   = iff.get_scalar("reff")
        degen  = iff.get_scalar("degen")
        self.const2 = (self.reff**2)/degen
        kappa = array(iff.get_array('feff.k'))
        chi = array(iff.get_array('feff.chi'))
        self.energy = self.E0 + const_energy*(kappa**2)
        self.pre_energy = arange((self.E0-500.0), (self.E0-0.5), 1.5)
        rd = reader.ReadXml()
        dic = rd.readInp(inp_file)
        stack, rep = rd.manageSTACK(dic)
        for i in range(len(stack)):
            if stack[i][0] == l_name:
                self.currentLayer = stack[i][2:]
                self.currentDensity = atof(split(stack[i][1][2])[1])
#         search element
        edge_session = CalcEdge(inp_file, self.E0, l_name)
        dmu = edge_session.Edge()
        self.rotot = 0.0
        for z in range(len(self.currentLayer)):
            if self.currentLayer[z][0] == el_name:
                element = Table_f1f2.Element(self.currentLayer[z][0])
                self.currentLayer[z][1] = split(self.currentLayer[z][1])
                roi = atof(self.currentLayer[z][1][0])*self.currentDensity
                Pmi = dabax.AtomicProperties(el_name,'AtomicMass')
                f20 = array(imag(element.f1f2Energy(self.energy)))
                pre_f2 = array(imag(element.f1f2Energy(self.pre_energy)))
        self.f2 = (self.energy*dmu*chi*Pmi)/(const_mu*roi) + f20
        self.tot_energy = concatenate((array(self.pre_energy),array(self.energy)))
        self.tot_f2 = concatenate((array(pre_f2),array(self.f2)))
        iff.ifeffit("erase @arrays, @scalars")
    def out_write(self):
        file = open('TEMP_files/f2', 'w')
        for z in range(len(self.tot_energy)):
            file.write(str(self.tot_energy[z])+'  '+str(self.tot_f2[z])+'\n')
        file.close()
        
class KK:
    def __init__(self, el_name, E0):
        self.E0 = E0
        self.zeta = dabax.AtomicProperties(el_name,'Z')
        file = open('TEMP_files/diffkk.inp', 'w')
        file.write('title = CARD out \n')
        file.write('out = TEMP_files/kk_out \n')
        file.write('xmu = TEMP_files/f2 \n')
        file.write('isfeff = false \n')
        file.write('encol = 1 \n')
        file.write('mucol = 2 \n')
        file.write('iz =   %i \n'%self.zeta)
        file.write('e0 =   %f \n'%self.E0)
        file.write('egrid = 1.0000 \n')
        file.write('ewidth = 0.0 \n')
        file.write('elow =    50.0000 \n')
        file.write('ehigh =   50.0000 \n')
        file.close()
    def Do_kk(self):
        diffkk.main()
    def New_f1f2(self):
        filename  = "TEMP_files/kk_out"
        data = loadtxt(filename, unpack=True)
        iff.put_array('a.f1', data[1])
        iff.put_array('a.f2', data[2])
        iff.ifeffit('b.f1 = smooth(a.f1)')
        iff.ifeffit('b.f2 = smooth(a.f2)')
        out_f1 = array(iff.get_array('b.f1'))
        out_f2 = array(iff.get_array('b.f2'))
        iff.ifeffit("erase @arrays, @scalars")
        for m in range(len(data[0])):
            if data[0][m] > self.E0:
                tok = m-1
                break
                
        temp_f1 = out_f1[tok-15:tok+15]
        temp_f10 = data[3][tok-15:tok+15]
        a = min(temp_f1)
        b = min(temp_f10)
        for i in range(len(data[0])):
            if out_f1[i] == a:
                pre = i
            if data[3][i] == b:
                post = i
                
        dele = data[0][pre]-data[0][post]
       # print 'dele', dele
        
        shift_e = data[0]-dele
        
        
        iff.put_array('a.e',data[0])
        iff.put_array('sh.e',shift_e)
        iff.put_array('a.f1',data[3])
        iff.put_array('a.f2',data[4])
        iff.ifeffit("sh.f1=rebin(a.e,a.f1,sh.e)")
        iff.ifeffit("sh.f2=rebin(a.e,a.f2,sh.e)")
        sh_f1 = array(iff.get_array('sh.f1'))
        sh_f2 = array(iff.get_array('sh.f2'))

        
        return data[0], out_f1, out_f2, sh_f1, sh_f2

class NWrite:
    def __init__(self, en, f1, f2, in_file, name, element_name):        
        self.tot_stoch = 0.0
        self.betai = []
        self.deltai = []
        self.f1 = []
        self.f2 = []
        self.beta = zeros(len(en))
        self.delta = ones(len(en))
        rd = reader.ReadXml()
        dic = rd.readInp(in_file)
        stack, rep = rd.manageSTACK(dic)
        for i in range(len(stack)):
            if stack[i][0] == name:
                self.currentLayer = stack[i][2:]
                self.currentDensity = atof(split(stack[i][1][2])[1])
   #                 append the f2s and calculates the other parameters
        for b in range(len(self.currentLayer)):
            f0_t = Table_f0.Element(self.currentLayer[b][0])
            f0 = array(f0_t.f0Energy(en))
            if self.currentLayer[b][0] == element_name:
                self.f2.append(f2)
                f0f1 = f0 + f1
                self.f1.append(f0f1)
            else:
                element = Table_f1f2.Element(self.currentLayer[b][0])
                self.f2.append(imag(element.f1f2Energy(en)))
                f0f1 = f0 + real(element.f1f2Energy(en))
                self.f1.append(f0f1)
            Pmi = dabax.AtomicProperties(self.currentLayer[b][0],'AtomicMass')
        for c in range(len(self.currentLayer)):
            Pmi = dabax.AtomicProperties(self.currentLayer[c][0],'AtomicMass')
            self.currentLayer[c][1] = split(self.currentLayer[c][1])
            betai = (atof(self.currentLayer[c][1][0])*const_beta*self.f2[c]*self.currentDensity)/(Pmi*(en**2))
            deltai = (atof(self.currentLayer[c][1][0])*const_beta*self.f1[c]*self.currentDensity)/(Pmi*(en**2))
            for y in range(len(betai)):
                self.beta[y] = self.beta[y]+ betai[y]
                self.delta[y] = self.delta[y] - deltai[y]
        self.beta = array(self.beta)
        self.delta = array(self.delta)
        self.en = en
        self.min_en = self.en[0]+10.0
        self.max_en = self.en[len(en)-1]-10.0
    def indexWrite(self, out_file):
        savetxt(out_file, transpose((self.en, self.delta, self.beta)), delimiter = ' ', fmt = '%13.12f')

        
class ReflConstructor():
    def __init__(self, l_name, min_en, max_en, in_file):
        self.index_e = 'TEMP_files/exafs_indexes' 
        self .index_0 = 'TEMP_files/indexes_0'
        lambdamin = 12398.52/max_en
        lambdamax = 12398.52/min_en
        lambdastep = (lambdamax - lambdamin)/2000.0
        self.lbd ='[%.4f,%.4f,%.4f]'% (lambdamin, lambdamax, lambdastep )
        rd = reader.ReadXml()
        dic = rd.readInp(in_file)
        scan = rd.manageSCAN(dic)
        new_scan = []
        new_flist = []
        for i in range(len(scan[1])):
            temp = split(scan[1][i])
            if temp[0] == '"wavelenghts_col"':
                f_i = temp[0]+' '+self.lbd
                new_flist.append(f_i)
            elif temp[0] == '"angles_col"':
                new_flist.append(scan[1][i])
            elif temp[0] == '"angle_factor"':
                new_flist.append(scan[1][i])
        new_scan.append(new_flist)
        stack, rep = rd.manageSTACK(dic)
        stack_exafs = []
        stack_0 = []
        for i in range(len(stack)):
            if stack[i][0] == l_name:
                stack_exafs.append([stack[i][0],stack[i][1],'iff',self.index_e])
                stack_0.append([stack[i][0],stack[i][1],'iff',self.index_0])
            else:
                stack_exafs.append(stack[i])
                stack_0.append(stack[i])    
#         create new .xml input files
#         first the stack with fine structure
        wt_e = writer.WriteXml()
        wt_e.createTF0("f0_WaasKirf.dat")
        wt_e.createTF12("f1f2_Sasaki.dat")
        sub_exafs = []
        for t in range(len(stack_exafs)):
            if stack_exafs[t][0] == 'sub':
                sub_exafs.append(stack_exafs[t])
                wt_e.createSTACK(sub_exafs, '1')
        if len(stack_exafs) > 1:
            stack_exafs = stack_exafs[1:]
            wt_e.createSTACK(stack_exafs, rep)
        wt_e.createSCAN(new_scan)
        wt_e.createFIT()
        wt_e.createCpW('TEMP_files/temp')
        wt_e.writeOutput('TEMP_files/temp_exafs.xml')
#         then the stack for the reference structure
        wt_0 = writer.WriteXml()
        wt_0.createTF0("f0_WaasKirf.dat")
        wt_0.createTF12("f1f2_Sasaki.dat")
        sub_0 = []
        for t in range(len(stack_0)):
            if stack_0[t][0] == 'sub':
                sub_0.append(stack_0[t])
                wt_0.createSTACK(sub_0, '1')
        if len(stack_0) > 1:
            stack_0 = stack_0[1:]
            wt_0.createSTACK(stack_0, rep)
        wt_0.createSCAN(new_scan)
        wt_0.createFIT()
        wt_0.createCpW('TEMP_files/temp')
        wt_0.writeOutput('TEMP_files/temp_0.xml')
    def CalculateChi(self, E0, mink, maxk, reff):
        self.E0 = E0
        s = ppmxml.get_script('TEMP_files/temp_exafs.xml')
        exec(s)
        self.lbd = array(fit2.scanlist[0][0])
        self.r_exafs = array(fit2.calculatedscan[0])
        s = ppmxml.get_script('TEMP_files/temp_0.xml')
        exec(s)
        self.r_0 = array(fit2.calculatedscan[0])
        chi_0 = self.r_exafs-self.r_0
        self.en = []
        self.chi = []
        for z in range(len(self.lbd)):
            self.en.append(12398.52/self.lbd[len(self.lbd)-1-z])
            self.chi.append(chi_0[len(self.lbd)-1-z])  
        self.chi = array(self.chi)
        self.en = array(self.en)

        
        new_erange = arange((self.E0 - round(self.E0-self.en[0])),(self.E0 - round(self.E0 - self.en[len(self.en)-1])), 1.0)
        iff.put_array('old.e',self.en)
        iff.put_array('old.chi', self.chi)
        iff.put_array('sp.energy', new_erange)
        iff.ifeffit('set sp.chi0 = rebin(old.e, old.chi, sp.energy)')
        newchi = array(iff.get_array('sp.chi0'))
        

# Slice of arrays respect to E0 and calculation of k array

        ered = "set sp.ered=slice(sp.energy,nofx(sp.energy,"+str(self.E0)+"),npts(sp.energy))"
        iff.ifeffit(ered)
        kappa = "set sp.k=sqrt((sp.ered-"+str(self.E0)+")/3.809984069)"
        iff.ifeffit(kappa)
        chitot = "sp.chi=slice(sp.chi0,nofx(sp.energy,"+str(self.E0)+"),npts(sp.energy))"
        iff.ifeffit(chitot)
        forward = "fftf(real=sp.chi, k=sp.k, kmin="+str(mink)+",kmax= "+str(maxk)+", dk=0,kwindow=hanning, kweight=2)"
        back = "fftr(real=sp.chir_re, imag=sp.chir_im, rmin=0.03,rmax="+str(reff*1.8)+", dr=0,rweight=0, rwindow=hanning)"
        iff.ifeffit(forward)
        iff.ifeffit(back)
        amp0 = array(iff.get_array("sp.chiq_mag"))
        self.pha = array(iff.get_array("sp.chiq_pha"))
        self.cu = array(iff.get_array("sp.q"))
        self.amp = amp0/(self.cu**2)
        self.amp[0] = amp0[0]/(self.cu[1]**2)
        iff.ifeffit("erase @arrays,@scalars")
    def Do_feff(self, f_file,camp,min_k,max_k,r_eff,costante):
        feff.feff_file(self.cu, self.amp, self.pha, f_file,camp, min_k,max_k,r_eff,costante)
        
        
if __name__ == '__main__':
    in_file = 'TEST/Au_105.xml'
    Pm = 196.9665
    E1 = 11919.0
    name = 'Au_layer'
    
    feff_file = 'TEST/feff0001.dat'
    out_feff = 'TEST/r_feff.dat'
    conta = 1
    element_name = 'Au'
    
    edge = CalcEdge(in_file, E1, name)
    mu0 =  edge.Edge()
    
    test = Edge_f2(feff_file, conta, element_name, in_file, E1, name)
    test.out_write()
    f2 = test.tot_f2
    en = test.tot_energy
    mink = test.mink
    maxk = test.maxk
    reff = test.reff
    const2 = test.const2
    kramers = KK(element_name, E1)
    kramers.Do_kk()
    new_en, new_f1, new_f2, f10, f20 = kramers.New_f1f2()
    exafs_index = NWrite(new_en, new_f1, new_f2, in_file, name, element_name)
    exafs_index.indexWrite('TEMP_files/exafs_indexes')
    index_0 = NWrite(new_en, f10, f20, in_file, name, element_name)
    index_0.indexWrite('TEMP_files/indexes_0')
    rfc = ReflConstructor(name,exafs_index.min_en,exafs_index.max_en,in_file)
    rfc.CalculateChi(E1, mink, maxk, reff)
    
    rfc.Do_feff(feff_file,out_feff,mink,maxk,reff,const2)
    
#     f0_t = Table_f0.Element("Ti")
#     f0 = array(f0_t.f0Energy(new_en))
#     
#     test_f1_el = Table_f1f2.Element("Ti")
#     test_f1 = array(real(test_f1_el.f1f2Energy(new_en)))
#     out_f1 = test_f1
#     savetxt('TEMP_files/test_f1', transpose((new_en, out_f1)), delimiter = ' ', fmt = '%13.12f')


