import sys, os
from PyQt4 import QtGui, QtCore
import shutil

from refl_constructor import *

class Refl(QtGui.QWidget):
    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)
        self.setWindowTitle('Refl')
        self.home = os.getenv("HOME")
        self.selectGroup = QtGui.QGroupBox()
        self.a_label = QtGui.QLabel('angle')
        self.angle = QtGui.QLineEdit()
        self.L_label = QtGui.QLabel('Select Layer:')
        self.El_label = QtGui.QLabel('Select element:')
        self.E0_label = QtGui.QLabel('Select E0 (eV): ')
        self.LayerSelect = QtGui.QComboBox()
        self.ElementSelect = QtGui.QLineEdit()
        self.E0Select = QtGui.QLineEdit()
        self.selectGroupLayout = QtGui.QGridLayout()
        self.selectGroupLayout.addWidget(self.a_label, 0,0)
        self.selectGroupLayout.addWidget(self.angle, 1,0)
        self.selectGroupLayout.addWidget(self.L_label, 2,0)
        self.selectGroupLayout.addWidget(self.El_label, 2,1)
        self.selectGroupLayout.addWidget(self.E0_label, 2,2)
        self.selectGroupLayout.addWidget(self.LayerSelect, 3,0)
        self.selectGroupLayout.addWidget(self.ElementSelect, 3,1)
        self.selectGroupLayout.addWidget(self.E0Select, 3,2)
        self.selectGroup.setLayout(self.selectGroupLayout)
        
        

        self.fileGroup = QtGui.QGroupBox()
        self.feffLabel = QtGui.QLabel('FEFF files path:')
        self.feffEdit = QtGui.QLineEdit()
        self.feffSelect = QtGui.QPushButton('SET')
        self.connect(self.feffSelect, QtCore.SIGNAL('clicked()'), self.selectFeffFolder)
        self.outLabel = QtGui.QLabel('Output Folder:')
        self.outEdit = QtGui.QLineEdit()
        self.outSelect = QtGui.QPushButton('SET')
        self.connect(self.outSelect, QtCore.SIGNAL('clicked()'), self.selectOutFolder)
        self.pathsLabel = QtGui.QLabel('Paths:')
        self.pathsSelect = QtGui.QSpinBox()
        self.pathsSelect.setValue(1)
        self.fileGroupLayout = QtGui.QGridLayout()
        self.fileGroupLayout.addWidget(self.feffLabel, 0,0)
        self.fileGroupLayout.addWidget(self.feffEdit, 0,1)
        self.fileGroupLayout.addWidget(self.feffSelect, 0,2)
        self.fileGroupLayout.addWidget(self.outLabel, 1,0)
        self.fileGroupLayout.addWidget(self.outEdit, 1,1)
        self.fileGroupLayout.addWidget(self.outSelect, 1,2)
        self.fileGroupLayout.addWidget(self.pathsLabel, 2,0)
        self.fileGroupLayout.addWidget(self.pathsSelect, 2,1)
        self.fileGroup.setLayout(self.fileGroupLayout)

        self.outGroup = QtGui.QGroupBox()
        self.goButton = QtGui.QPushButton('GO!')
        self.outGroupLayout = QtGui.QHBoxLayout()
        self.outGroupLayout.addWidget(self.goButton)
        self.outGroup.setLayout(self.outGroupLayout)
        
        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.selectGroup)
        layout.addWidget(self.fileGroup)
        layout.addWidget(self.outGroup)
        self.setLayout(layout)

    def selectFeffFolder(self):
        if not str(self.feffEdit.text()) == '':
            self.home = str(self.feffEdit.text())
        fileName = QtGui.QFileDialog.getExistingDirectory(self, "Open Directory",self.home , QtGui.QFileDialog.ShowDirsOnly)
        if len(fileName) == 0:
            return
        fileName=str(fileName)
        self.home = str(fileName)
        self.feffEdit.setText(fileName)

    def selectOutFolder(self):
        if not str(self.outEdit.text()) == '':
            self.home = str(self.outEdit.text())
        fileName = QtGui.QFileDialog.getExistingDirectory(self, "Open Directory",self.home , QtGui.QFileDialog.ShowDirsOnly)
        if len(fileName) == 0:
            return
        fileName=str(fileName)
        self.home = str(fileName)
        self.outEdit.setText(fileName)
        
    def Calculate(self):
        edge = CalcEdge('TEMP_files/feff.xml', atof(str(self.E0Select.text())), str(self.LayerSelect.currentText()))
        self.mu0 = edge.Edge()
        kramers = KK(str(self.ElementSelect.text()), atof(str(self.E0Select.text())))
        conta  = 0
        diff   = 0
        cicli = self.pathsSelect.value()
        for i in range(cicli):
            conta = i + 1
            inp_feff = '%s/feff%04d.dat' % (str(self.feffEdit.text()),conta)
            out_feff = '%s/feff%04d.dat' % (str(self.outEdit.text()),conta)
            try:
                file = open(inp_feff, 'r')
                file.close()
            except:
                diff = diff + 1
                if diff == cicli :
                    msg = 'No feffxxxx.dat files found in selected directory! '
                    self.emit(QtCore.SIGNAL('add_log'), msg)
                    break
                continue
            edge_f2 = Edge_f2(inp_feff, conta, str(self.ElementSelect.text()), 'TEMP_files/feff.xml', atof(str(self.E0Select.text())), str(self.LayerSelect.currentText()))
            edge_f2.out_write()
            f2 = edge_f2.tot_f2
            en = edge_f2.tot_energy
            mink = edge_f2.mink
            maxk = edge_f2.maxk
            reff = edge_f2.reff
            const2 = edge_f2.const2
            kramers.Do_kk()
            new_en, new_f1, new_f2, f10, f20 = kramers.New_f1f2()
            
            exafs_index = NWrite(new_en, new_f1, new_f2, 'TEMP_files/feff.xml', str(self.LayerSelect.currentText()), str(self.ElementSelect.text()))
            exafs_index.indexWrite('TEMP_files/exafs_indexes')
            index_0 = NWrite(new_en, f10, f20, 'TEMP_files/feff.xml', str(self.LayerSelect.currentText()), str(self.ElementSelect.text()))
            index_0.indexWrite('TEMP_files/indexes_0')
            rfc = ReflConstructor(str(self.LayerSelect.currentText()),exafs_index.min_en,exafs_index.max_en,'TEMP_files/feff.xml')
            rfc.CalculateChi(atof(str(self.E0Select.text())), mink, maxk, reff)
            
            rfc.Do_feff(inp_feff,out_feff,mink,maxk,reff,const2)
        self.numero = cicli-diff
        
if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    qb = Refl()
    qb.show()
    sys.exit(app.exec_())
